/*	Assignment 2 -- Clipping
================================================================================
	loader.cpp

		Model loader helper functions
================================================================================
*/

////////////////////////////////////////////////////////////////////////////////
//// Includes //////////////////////////////////////////////////////////////////
#include "loader.h"

#include <iostream>
#include <fstream>
#include <cstring>
//#include <string>
using namespace std;

////////////////////////////////////////////////////////////////////////////////
//// Functions /////////////////////////////////////////////////////////////////
//--------------------------------
// LoadModel
//		Loads a poly model from the specified filename.
//	Returns a pointer to the model on success, or NULL on failure.
SPolyModel *LoadPolyModel(const char szFilename[])
{
	SPolyModel *pModel;
	SVertex	*pCurrVert;
	SFace	*pCurrFace;
//	string sBuff;
	char	szBuff[80];
	ifstream fin;

	int	iCurrVert, iCurrFace;

	fin.open(szFilename);

	if (fin.is_open())
	{
		pModel = new SPolyModel;
		fin >> szBuff;
		if (!stricmp(szBuff, "vertices:"))
			fin >> pModel->iNumVerts;
		else
		{
			cout << "Error: Model \"" << szFilename << "\" not of expected format." << endl;
			delete pModel;
			return NULL;
		}

		fin >> szBuff;
		if (!stricmp(szBuff, "faces:"))
			fin >> pModel->iNumFaces;
		else
		{
			cout << "Error: Model \"" << szFilename << "\" not of expected format." << endl;
			delete pModel;
			return NULL;
		}

		pModel->pVerts = new SVertex[pModel->iNumVerts];
		pModel->pFaces = new SFace[pModel->iNumFaces];

		iCurrVert = 0;
		iCurrFace = 0;
		while (fin >> szBuff)
		{
			if (szBuff[0] == 'v')
			{
				pCurrVert = &pModel->pVerts[iCurrVert++];
				fin	>> pCurrVert->vPosition[0]
					>> pCurrVert->vPosition[1]
					>> pCurrVert->vPosition[2]
					>> pCurrVert->vNormal[0]
					>> pCurrVert->vNormal[1]
					>> pCurrVert->vNormal[2];
			}
			else if (szBuff[0] == 'f')
			{
				pCurrFace = &pModel->pFaces[iCurrFace++];
				fin	>> pCurrFace->iVerts[0]
					>> pCurrFace->iVerts[1]
					>> pCurrFace->iVerts[2];
				--pCurrFace->iVerts[0];
				--pCurrFace->iVerts[1];
				--pCurrFace->iVerts[2];
			}
			else
			{
				cout << "Error: Model \"" << szFilename << "\" not of expected format." << endl;
				FreePolyModel(pModel);
				return NULL;
			}
		}
		fin.close();
		return pModel;
	}
	else
	{
		cout << "Error: Could not load model \"" << szFilename << "\"" << endl;
		return NULL;
	}
}

//--------------------------------
// FreePolyModel
//		Deallocates the specified model.
void	FreePolyModel(SPolyModel *pModel)
{
	if (pModel)
	{
		delete [] pModel->pVerts;
		delete [] pModel->pFaces;
		delete pModel;
	}
}
