////////////////////////////////////////////////////////////////////////////////
//// Includes //////////////////////////////////////////////////////////////////
#include <stdlib.h>

#include "student.h"
#include "vec.h"
#include "bmp.h"

////////////////////////////////////////////////////////////////////////////////
//// Functions /////////////////////////////////////////////////////////////////
CNPRender::CNPRender()
{
    Image hatch_img[4];

    LoadBMP("data/xhatch1.bmp", &hatch_img[0]);
    LoadBMP("data/xhatch2.bmp", &hatch_img[1]);
    LoadBMP("data/xhatch3.bmp", &hatch_img[2]);
    LoadBMP("data/xhatch4.bmp", &hatch_img[3]);

    // Create Texture   
    glGenTextures(1, &this->hatch_tex);
    glBindTexture(GL_TEXTURE_2D, this->hatch_tex);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);

    for (int i=0; i<4; ++i)
    {
        glTexImage2D(   GL_TEXTURE_2D, i, 3, hatch_img[i].sizeX, hatch_img[i].sizeY, 0, 
                        GL_RGBA, GL_UNSIGNED_BYTE, hatch_img[i].pixels);
    }

    free(hatch_img[0].pixels);
    free(hatch_img[1].pixels);
    free(hatch_img[2].pixels);
    free(hatch_img[3].pixels);
}

void CNPRender::DrawModel(SPolyModel *model, float eye_pos[])
{
    Vec3f eye, vert, norm, eye_dir;
    float dot;
    float color;

    // Assume the view transform has already been applied
    glColor3f(1.0f, 1.0f, 1.0f);

    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

    glEnable(GL_TEXTURE_2D);
    glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glEnable(GL_TEXTURE_GEN_S);
    glEnable(GL_TEXTURE_GEN_T);
    glBindTexture(GL_TEXTURE_2D, this->hatch_tex);

    eye.x = eye_pos[0];
    eye.y = eye_pos[1];
    eye.z = eye_pos[2];

    color = 1.0f;

    glBegin(GL_TRIANGLES);
	for (int iFace=0; iFace<model->iNumFaces; ++iFace)
	{
        memcpy(&vert, model->pVerts[model->pFaces[iFace].iVerts[0]].vPosition, 3*sizeof(float));
        memcpy(&norm, model->pVerts[model->pFaces[iFace].iVerts[0]].vNormal, 3*sizeof(float));
        glColor3f(color, color, color);
		glNormal3fv((float *)&norm);
		glVertex3fv((float *)&vert);

        memcpy(&vert, model->pVerts[model->pFaces[iFace].iVerts[1]].vPosition, 3*sizeof(float));
        memcpy(&norm, model->pVerts[model->pFaces[iFace].iVerts[1]].vNormal, 3*sizeof(float));
        glColor3f(color, color, color);
		glNormal3fv((float *)&norm);
		glVertex3fv((float *)&vert);

        memcpy(&vert, model->pVerts[model->pFaces[iFace].iVerts[2]].vPosition, 3*sizeof(float));
        memcpy(&norm, model->pVerts[model->pFaces[iFace].iVerts[2]].vNormal, 3*sizeof(float));
        glColor3f(color, color, color);
		glNormal3fv((float *)&norm);
		glVertex3fv((float *)&vert);
	}
	glEnd();

/*
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
*/
}